@@ -12,11 +12,19 @@ mclustboot.intern<-function(don, G = NULL, modelNames = NULL, verbose = FALSE){
                     modelName = "VVV",
                     use = "STD")
   res.mclust <- Mclust(don, G = G, modelNames = modelNames, verbose = verbose, initialization = list(hcPairs = res.init.hc))
+  if (is.null(res.mclust)) {
+    res.mclust <- Mclust(don, G = G, modelNames = "VVI", 
+                         verbose = verbose, initialization = list(hcPairs = res.init.hc))
+  }
   res.mclust.boot <- MclustBootstrap(object = res.mclust,nboot=1,type="bs",verbose=verbose)
   res.mclust$param$pro<-res.mclust.boot$pro
   res.mclust$param$mean<-matrix(res.mclust.boot$mean, nrow=dim(res.mclust.boot$mean)[3], byrow=TRUE)
+  if(nchar(modelNames)==3){
   res.mclust$param$variance$sigma<-array(res.mclust.boot$variance,
                                          dim=dim(res.mclust$param$variance$sigma),
                                          dimnames = dimnames(res.mclust$param$variance$sigma))
+  }else if (nchar(modelNames)==1){
+    res.mclust$param$variance$sigma<-res.mclust$param$variance$sigma
+  }
   return(res.mclust)
-}
\ No newline at end of file
+}
